/*
This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

In jurisdictions that recognize copyright laws, the author or authors
of this software dedicate any and all copyright interest in the
software to the public domain. We make this dedication for the benefit
of the public at large and to the detriment of our heirs and
successors. We intend this dedication to be an overt act of
relinquishment in perpetuity of all present and future rights to this
software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

For more information, please refer to <http://unlicense.org/>
*/
//============================================================================
// CFAG12864C Demonstration code
// ST7920 LCD Controller
// Atmel ATMega32 processor @ 16MHz
// WinAVR / AVR GCC + Atmel AVR Studio
// Written by Brent A. Crosby
// Modified by Max Roberg
// Crystalfontz America, Inc.
//============================================================================
#include <avr/io.h>
#include <util/delay.h>
#include <avr/iom2561.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "utils.h"
#include "low_lcd.h"
#include "logo_screen.h"
#include "MMC_SD.h"

//============================================================================
//This is the display memory.
volatile unsigned char display[32][32];  //1024 bytes
unsigned char displaytext[2][32]={
//									   1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32
									{'0','1','2','3','4','5','6','7','8','9','0','A','B','C','D','E','F','W','X','Y','Z',  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11},
									{'G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V', 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27}
								};
unsigned int chinesetext[2][16]={
									{0xA6B7, 0xA6B6, 0xA6B5, 0xA6B4, 0xA6B3, 0xA6B2, 0xA6B1, 0xA6B0, 0xA757, 0xA756, 0xA755, 0xA754, 0xA753, 0xA752, 0xA751, 0xA750},
									{0xA6BF, 0xA6BE, 0xA6BD, 0xA6BC, 0xA6BB, 0xA6BA, 0xA6B9, 0xA6B8, 0xA75F, 0xA75E, 0xA75D, 0xA75C, 0xA75B, 0xA75A, 0xA759, 0xA758}
								};
//============================================================================
volatile unsigned char display_status;

//It consists of 5 consecutive '1'(sync character).
// Serial transfer counter will be reset and synchronized.
// Following 2 bits for read/write (RW) and register/data select (RS).
// Last 4 bits is filled by '0'
#define SPI_COMMAND_HEADER (0xF8)
#define SPI_DATA_HEADER (0xFA)

//You write the header (above) then write a nibble at a time justified to the
// upper 4 bits of the SPI stream
//============================================================================
void write_command(unsigned char command)
{
	SPI_WriteByte(SPI_COMMAND_HEADER);	// send command header
	SPI_WriteByte(command&0xF0);	// send high nibble
	_delay_us(250);
	SPI_WriteByte((command<<4)&0xF0);	// send low nibble
	_delay_us(750);
}
//----------------------------------------------------------------------------
void write_data(unsigned char data)
{
	SPI_WriteByte(SPI_DATA_HEADER);	// send data header
	SPI_WriteByte(data&0xF0);	// send high nibble
	_delay_us(250);
	SPI_WriteByte((data<<4)&0xF0);	// send low nibble
	_delay_us(750);
}
//----------------------------------------------------------------------------
void write_data_16(unsigned short data)
{
	SPI_WriteByte(SPI_DATA_HEADER);	// send data header
	_delay_us(250);

	SPI_WriteByte((data&0xF000)>>8);	// send high nibble of MSB
	_delay_us(250);
	SPI_WriteByte((data&0x0F00)>>4);	// send low nibble of MSB
	_delay_us(750);
	SPI_WriteByte(data&0x00F0);			// send high nibble of LSB
	_delay_us(750);
	SPI_WriteByte((data&0x000F)<<4);	// send low nibble of LSB
	_delay_us(750);

/*
	SPI_WriteByte(data>>8);
	_delay_us(750);
	SPI_WriteByte(data);
*/

/*
	SPI_WriteByte(data&0xF000);	// send high nibble
	_delay_us(250);
	SPI_WriteByte((data<<4)&0xF000);	// send low nibble
	_delay_us(750);
	SPI_WriteByte((data<<8)&0xF000);	// send low nibble
	_delay_us(750);
	SPI_WriteByte((data<<12)&0xF000);	// send low nibble
	_delay_us(750);
*/
}
/*
void write_data_16(unsigned short data)
{
  //Go back to port A as an output
  LCD_DATA_DDR = LCD_DATA_ALL_OUTPUTS;
  //Write the data to the output latches of port A
  LCD_DATA_PORT = data>>8;
  //Talk to the data register.
  SET_RS;
  //Tell the LCD that we are writing to it.
  CLR_RW;
  SET_E;
  CLR_E;
  _delay_us(850);
  SET_E;
  LCD_DATA_PORT = data;
  CLR_E;
  _delay_us(850);
  SET_E;
}
*/
//----------------------------------------------------------------------------
void show_text(void)
{
     register unsigned char page_address;
     register unsigned char column_address;

     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
     write_command(0x38);
//     write_command(0x08);
     write_command(0x0A);  // cursor on
     write_command(0x01);
     write_command(0x06);
     write_command(0x0F);
     write_command(0x01);

	for (page_address = 0; page_address < 2; page_address++)
	{

		if(page_address==1)
			write_command(0x90);

		for (column_address = 0; column_address <33; column_address++)
		{
			write_data(displaytext[page_address][column_address]);
		}
	}
}

//----------------------------------------------------------------------------
void show_chinese_text(void)
{
	register unsigned char page_address;
	register unsigned char column_address;

	write_command(0x38);
	write_command(0x38);
	write_command(0x38);
	write_command(0x38);
	write_command(0x08);
//	write_command(0x0A);  // cursor on
	write_command(0x01);
	write_command(0x06);
	write_command(0x0F);
	write_command(0x01);

	write_command(0x80);

	for (page_address = 0; page_address < 2; page_address++)
	{
		if(page_address==1)
			write_command(0x90);

		for (column_address = 0; column_address <17; column_address++)
		{
			write_data_16(chinesetext[page_address][column_address]);
		}
	}
}

//============================================================================
void UpdateLCD(void)
{
	register volatile unsigned char *data;
	register unsigned char page_address;
	register unsigned char column_address;

	data = &display[0][0];


	//Initialize our source data pointers.
	for (page_address = 0; page_address < 32; page_address++)
	{
		//Set Y address
		write_command(page_address | 0x80);
		//Set X address 0 (right to left)
		write_command(0x80);

		for (column_address = 0; column_address <32; column_address++)
		{
			write_data(*data++);
		}
	}
}

void ResetLCD(void)
{
	CLR_RESET
	_delay_us(6500);
	_delay_us(6500);    
	_delay_us(6500);
	_delay_us(6500);
	_delay_us(6500);
	_delay_us(6500);
	SET_RESET
}

//============================================================================
void InitializeLCD(void)
{
	/* Take the LCD out of reset (if reset was low)
	*/
	ResetLCD();

	_delay_us(6500);

	//Display on
	write_command(0x30);
	write_command(0x0C);	// display ON, cursor OFF, blink OFF
	write_command(0x01);	// Display clear 
	write_command(0x06);	// Cursor moves right, no display shift
	write_command(0x30);	
	write_command(0x32);
	write_command(0x36);

}
//============================================================================
